@@ -172,8 +172,8 @@ static int string_to_int(const char *s)
 	l = strtol(s, &p, 0);
 	if (*p || l == LONG_MIN || l == LONG_MAX || l < 0 || l > INT_MAX)
 		return -1;
-	else
-		return (int) l;
+
+	return (int) l;
 }
 
 /* Do we really really want to check this fs? */
@@ -398,7 +398,6 @@ done:
 		inst->lockpath = NULL;
 	}
 	free(diskpath);
-	return;
 }
 
 static void unlock_disk(struct fsck_instance *inst)
@@ -425,7 +424,6 @@ static void free_instance(struct fsck_instance *i)
 	free(i->lockpath);
 	mnt_unref_fs(i->fs);
 	free(i);
-	return;
 }
 
 static struct libmnt_fs *add_dummy_fs(const char *device)
@@ -821,10 +819,10 @@ static struct fsck_instance *wait_one(int flags)
 		for (inst2 = instance_list; inst2; inst2 = inst2->next) {
 			if (inst2->flags & FLAG_DONE)
 				continue;
-			if (strcmp(inst2->type, "ext2") &&
+			if (strcmp(inst2->type, "ext2") != 0 &&
 			    strcmp(inst2->type, "ext3") &&
-			    strcmp(inst2->type, "ext4") &&
-			    strcmp(inst2->type, "ext4dev"))
+			    strcmp(inst2->type, "ext4") != 0 &&
+			    strcmp(inst2->type, "ext4dev") != 0)
 				continue;
 			/*
 			 * If we've just started the fsck, wait a tiny
@@ -905,8 +903,8 @@ static int fsck_device(struct libmnt_fs *fs, int interactive)
 
 	if (type && strcmp(type, "auto") != 0)
 		;
-	else if (fstype && strncmp(fstype, "no", 2) &&
-	    strncmp(fstype, "opts=", 5) && strncmp(fstype, "loop", 4) &&
+	else if (fstype && strncmp(fstype, "no", 2) != 0 &&
+	    strncmp(fstype, "opts=", 5) != 0 && strncmp(fstype, "loop", 4) != 0 &&
 	    !strchr(fstype, ','))
 		type = fstype;
 	else
