@@ -1,7 +1,9 @@
 /*
  * Fundamental C definitions.
+ *
+ * No copyright is claimed.  This code is in the public domain; do with
+ * it what you wish.
  */
-
 #ifndef UTIL_LINUX_C_H
 #define UTIL_LINUX_C_H
 
@@ -64,7 +66,7 @@
 
 /*
  * It evaluates to 1 if the attribute/feature is supported by the current
- * compilation targed. Fallback for old compilers.
+ * compilation target. Fallback for old compilers.
  */
 #ifndef __has_attribute
   #define __has_attribute(x) 0
@@ -348,6 +350,7 @@ static inline int xusleep(useconds_t usec)
 #define USAGE_OPTIONS    _("\nOptions:\n")
 #define USAGE_FUNCTIONS  _("\nFunctions:\n")
 #define USAGE_COMMANDS   _("\nCommands:\n")
+#define USAGE_ARGUMENTS   _("\nArguments:\n")
 #define USAGE_COLUMNS    _("\nAvailable output columns:\n")
 #define USAGE_SEPARATOR    "\n"
 
@@ -360,6 +363,11 @@ static inline int xusleep(useconds_t usec)
 		, " -h, --help",    USAGE_OPTSTR_HELP \
 		, " -V, --version", USAGE_OPTSTR_VERSION
 
+#define USAGE_ARG_SEPARATOR    "\n"
+#define USAGE_ARG_SIZE(_name) \
+		_(" %s arguments may be followed by the suffixes for\n" \
+		  "   GiB, TiB, PiB, EiB, ZiB, and YiB (the \"iB\" is optional)\n"), _name
+
 #define USAGE_MAN_TAIL(_man)   _("\nFor more details see %s.\n"), _man
 
 #define UTIL_LINUX_VERSION _("%s from %s\n"), program_invocation_short_name, PACKAGE_STRING
@@ -369,15 +377,6 @@ static inline int xusleep(useconds_t usec)
 		exit(eval); \
 })
 
-/*
- * scanf modifiers for "strings allocation"
- */
-#ifdef HAVE_SCANF_MS_MODIFIER
-#define UL_SCNsA	"%ms"
-#elif defined(HAVE_SCANF_AS_MODIFIER)
-#define UL_SCNsA	"%as"
-#endif
-
 /*
  * seek stuff
  */
