@@ -11,34 +11,7 @@
 #include <stdio.h>
 #include <unistd.h>
 
-#define UL_COLOR_RESET		"\033[0m"
-#define UL_COLOR_BOLD		"\033[1m"
-#define UL_COLOR_HALFBRIGHT	"\033[2m"
-#define UL_COLOR_UNDERSCORE	"\033[4m"
-#define UL_COLOR_BLINK		"\033[5m"
-#define UL_COLOR_REVERSE	"\033[7m"
-
-/* Standard colors */
-#define UL_COLOR_BLACK		"\033[30m"
-#define UL_COLOR_RED		"\033[31m"
-#define UL_COLOR_GREEN		"\033[32m"
-#define UL_COLOR_BROWN		"\033[33m"	/* well, brown */
-#define UL_COLOR_BLUE		"\033[34m"
-#define UL_COLOR_MAGENTA	"\033[35m"
-#define UL_COLOR_CYAN		"\033[36m"
-#define UL_COLOR_GRAY		"\033[37m"
-
-/* Bold variants */
-#define UL_COLOR_DARK_GRAY	"\033[1;30m"
-#define UL_COLOR_BOLD_RED	"\033[1;31m"
-#define UL_COLOR_BOLD_GREEN	"\033[1;32m"
-#define UL_COLOR_BOLD_YELLOW	"\033[1;33m"
-#define UL_COLOR_BOLD_BLUE	"\033[1;34m"
-#define UL_COLOR_BOLD_MAGENTA	"\033[1;35m"
-#define UL_COLOR_BOLD_CYAN	"\033[1;36m"
-
-#define UL_COLOR_WHITE		"\033[1;37m"
-
+#include "color-names.h"
 
 /* --color[=WHEN] */
 enum colortmode {
@@ -50,10 +23,10 @@ enum colortmode {
 	__UL_NCOLORMODES	/* last */
 };
 
-#ifdef USE_COLORS_BY_DEFAUL
-# define USAGE_COLORS_DEFAULT	N_("colors enabled by default")
+#ifdef USE_COLORS_BY_DEFAULT
+# define USAGE_COLORS_DEFAULT	_("colors are enabled by default")
 #else
-# define USAGE_COLORS_DEFAULT   N_("colors disabled by default")
+# define USAGE_COLORS_DEFAULT   _("colors are disabled by default")
 #endif
 
 extern int colormode_from_string(const char *str);
@@ -65,6 +38,9 @@ extern int colors_init(int mode, const char *util_name);
 /* Returns 1 or 0 */
 extern int colors_wanted(void);
 
+/* Returns UL_COLORMODE_* */
+extern int colors_mode(void);
+
 /* temporary enable/disable colors */
 extern void colors_off(void);
 extern void colors_on(void);
@@ -94,8 +70,4 @@ static inline void color_disable(void)
 	color_fdisable(stdout);
 }
 
-/* converts "red" to UL_COLOR_RED, etc. */
-extern const char *color_sequence_from_colorname(const char *str);
-
-
 #endif /* UTIL_LINUX_COLORS_H */
