@@ -195,6 +195,20 @@ static const char *get_absdir(struct path_cxt *pc)
 	return pc->path_buffer;
 }
 
+int ul_path_is_accessible(struct path_cxt *pc)
+{
+	const char *path;
+	assert(pc);
+
+	if (pc->dir_fd >= 0)
+		return 1;
+
+	path = get_absdir(pc);
+	if (!path)
+		return 0;
+	return access(path, F_OK) == 0;
+}
+
 int ul_path_get_dirfd(struct path_cxt *pc)
 {
 	assert(pc);
@@ -218,7 +232,7 @@ void ul_path_close_dirfd(struct path_cxt *pc)
 	assert(pc);
 
 	if (pc->dir_fd >= 0) {
-		DBG(CXT, ul_debugobj(pc, "closing dir: '%s'", pc->dir_path));
+		DBG(CXT, ul_debugobj(pc, "closing dir"));
 		close(pc->dir_fd);
 		pc->dir_fd = -1;
 	}
@@ -289,19 +303,27 @@ char *ul_path_get_abspath(struct path_cxt *pc, char *buf, size_t bufsz, const ch
 
 int ul_path_access(struct path_cxt *pc, int mode, const char *path)
 {
-	int dir, rc;
+	int rc;
 
-	dir = ul_path_get_dirfd(pc);
-	if (dir < 0)
-		return dir;
+	if (!pc) {
+		rc = access(path, mode);
+		DBG(CXT, ul_debug("access '%s' [no context, rc=%d]", path, rc));
+	} else {
+		int dir = ul_path_get_dirfd(pc);
+		if (dir < 0)
+			return dir;
+		if (*path == '/')
+			path++;
 
-	DBG(CXT, ul_debugobj(pc, "access: '%s'", path));
-	rc = faccessat(dir, path, mode, 0);
-
-	if (rc && errno == ENOENT
-	    && pc->redirect_on_enoent
-	    && pc->redirect_on_enoent(pc, path, &dir) == 0)
 		rc = faccessat(dir, path, mode, 0);
+
+		if (rc && errno == ENOENT
+		    && pc->redirect_on_enoent
+		    && pc->redirect_on_enoent(pc, path, &dir) == 0)
+			rc = faccessat(dir, path, mode, 0);
+
+		DBG(CXT, ul_debugobj(pc, "access: '%s' [rc=%d]", path, rc));
+	}
 	return rc;
 }
 
@@ -317,6 +339,32 @@ int ul_path_accessf(struct path_cxt *pc, int mode, const char *path, ...)
 	return !p ? -errno : ul_path_access(pc, mode, p);
 }
 
+int ul_path_stat(struct path_cxt *pc, struct stat *sb, const char *path)
+{
+	int rc;
+
+	if (!pc) {
+		rc = stat(path, sb);
+		DBG(CXT, ul_debug("stat '%s' [no context, rc=%d]", path, rc));
+	} else {
+		int dir = ul_path_get_dirfd(pc);
+		if (dir < 0)
+			return dir;
+		if (*path == '/')
+			path++;
+
+		rc = fstatat(dir, path, sb, 0);
+
+		if (rc && errno == ENOENT
+		    && pc->redirect_on_enoent
+		    && pc->redirect_on_enoent(pc, path, &dir) == 0)
+			rc = fstatat(dir, path, sb, 0);
+
+		DBG(CXT, ul_debugobj(pc, "stat '%s' [rc=%d]", path, rc));
+	}
+	return rc;
+}
+
 int ul_path_open(struct path_cxt *pc, int flags, const char *path)
 {
 	int fd;
@@ -330,6 +378,9 @@ int ul_path_open(struct path_cxt *pc, int flags, const char *path)
 		if (dir < 0)
 			return dir;
 
+		if (*path == '/')
+			path++;
+
 		fdx = fd = openat(dir, path, flags);
 
 		if (fd < 0 && errno == ENOENT
@@ -503,6 +554,9 @@ ssize_t ul_path_readlink(struct path_cxt *pc, char *buf, size_t bufsiz, const ch
 	if (dirfd < 0)
 		return dirfd;
 
+	if (*path == '/')
+		path++;
+
 	return readlinkat(dirfd, path, buf, bufsiz);
 }
 
@@ -610,9 +664,10 @@ int ul_path_read_buffer(struct path_cxt *pc, char *buf, size_t bufsz, const char
 
 	/* Remove tailing newline (usual in sysfs) */
 	if (rc > 0 && *(buf + rc - 1) == '\n')
-		--rc;
+		buf[--rc] = '\0';
+	else
+		buf[rc - 1] = '\0';
 
-	buf[rc] = '\0';
 	return rc;
 }
 
