@@ -1,17 +1,11 @@
 /*
  * Copyright (C) 2011 Davidlohr Bueso <dave@gnu.org>
+ * Copyright (C) 2011-2020 Karel Zak <kzak@redhat.com>
  *
  * procutils.c: General purpose procfs parsing utilities
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Library Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Library Public License for more details.
+ * No copyright is claimed.  This code is in the public domain; do with
+ * it what you wish.
  */
 
 #include <stdio.h>
@@ -241,7 +235,7 @@ int proc_next_pid(struct proc_processes *ps, pid_t *pid)
 	return 0;
 }
 
-#ifdef TEST_PROGRAM
+#ifdef TEST_PROGRAM_PROCUTILS
 
 static int test_tasks(int argc, char *argv[])
 {
@@ -305,4 +299,4 @@ int main(int argc, char *argv[])
 
 	return EXIT_FAILURE;
 }
-#endif /* TEST_PROGRAM */
+#endif /* TEST_PROGRAM_PROCUTILS */
