@@ -18,6 +18,7 @@
 #include <stdint.h>
 #include <stddef.h>
 #include <limits.h>
+#include <inttypes.h>
 
 #include "partitions.h"
 #include "crc32.h"
@@ -55,7 +56,7 @@ struct gpt_header {
 
 	uint64_t	my_lba;			/* location of this header copy */
 	uint64_t	alternate_lba;		/* location of the other header copy */
-	uint64_t	first_usable_lba;	/* lirst usable LBA for partitions */
+	uint64_t	first_usable_lba;	/* first usable LBA for partitions */
 	uint64_t	last_usable_lba;	/* last usable LBA for partitions */
 
 	efi_guid_t	disk_guid;		/* disk UUID */
@@ -102,9 +103,10 @@ struct gpt_entry {
 /*
  * EFI uses crc32 with ~0 seed and xor's with ~0 at the end.
  */
-static inline uint32_t count_crc32(const unsigned char *buf, size_t len)
+static inline uint32_t count_crc32(const unsigned char *buf, size_t len,
+				   size_t exclude_off, size_t exclude_len)
 {
-	return (crc32(~0L, buf, len) ^ ~0L);
+	return (ul_crc32_exclude_offset(~0L, buf, len, exclude_off, exclude_len) ^ ~0L);
 }
 
 static inline unsigned char *get_lba_buffer(blkid_probe pr,
@@ -132,7 +134,7 @@ static void swap_efi_guid(efi_guid_t *uid)
 
 static int last_lba(blkid_probe pr, uint64_t *lba)
 {
-	blkid_loff_t sz = blkid_probe_get_size(pr);
+	uint64_t sz = blkid_probe_get_size(pr);
 	unsigned int ssz = blkid_probe_get_sectorsize(pr);
 
 	if (sz < ssz)
@@ -154,7 +156,7 @@ static int last_lba(blkid_probe pr, uint64_t *lba)
  * (due DOS PT limitations).
  *
  * Note that the PMBR detection is optional (enabled by default) and could be
- * disabled by BLKID_PARTS_FOPCE_GPT flag (see also blkid_paertitions_set_flags()).
+ * disabled by BLKID_PARTS_FOPCE_GPT flag (see also blkid_partitions_set_flags()).
  */
 static int is_pmbr_valid(blkid_probe pr, int *has)
 {
@@ -165,8 +167,8 @@ static int is_pmbr_valid(blkid_probe pr, int *has)
 
 	if (has)
 		*has = 0;
-	if (flags & BLKID_PARTS_FORCE_GPT)
-		goto ok;			/* skip PMBR check */
+	else if (flags & BLKID_PARTS_FORCE_GPT)
+		return 1;			/* skip PMBR check */
 
 	data = blkid_probe_get_sector(pr, 0);
 	if (!data) {
@@ -179,8 +181,10 @@ static int is_pmbr_valid(blkid_probe pr, int *has)
 		goto failed;
 
 	for (i = 0, p = mbr_get_partition(data, 0); i < 4; i++, p++) {
-		if (p->sys_ind == MBR_GPT_PARTITION)
+		if (p->sys_ind == MBR_GPT_PARTITION) {
+			DBG(LOWPROBE, ul_debug(" #%d valid PMBR partition", i + 1));
 			goto ok;
+		}
 	}
 failed:
 	return 0;
@@ -207,13 +211,15 @@ static struct gpt_header *get_gpt_header(
 				uint64_t lastlba)
 {
 	struct gpt_header *h;
-	uint32_t crc, orgcrc;
+	uint32_t crc;
 	uint64_t lu, fu;
-	size_t esz;
+	uint64_t esz;
 	uint32_t hsz, ssz;
 
 	ssz = blkid_probe_get_sectorsize(pr);
 
+	DBG(LOWPROBE, ul_debug(" checking for GPT header at %"PRIu64, lba));
+
 	/* whole sector is allocated for GPT header */
 	h = (struct gpt_header *) get_lba_buffer(pr, lba, ssz);
 	if (!h)
@@ -231,12 +237,11 @@ static struct gpt_header *get_gpt_header(
 		return NULL;
 
 	/* Header has to be verified when header_crc32 is zero */
-	orgcrc = h->header_crc32;
-	h->header_crc32 = 0;
-	crc = count_crc32((unsigned char *) h, hsz);
-	h->header_crc32 = orgcrc;
+	crc = count_crc32((unsigned char *) h, hsz,
+			offsetof(struct gpt_header, header_crc32),
+			sizeof(h->header_crc32));
 
-	if (crc != le32_to_cpu(orgcrc)) {
+	if (crc != le32_to_cpu(h->header_crc32)) {
 		DBG(LOWPROBE, ul_debug("GPT header corrupted"));
 		return NULL;
 	}
@@ -264,17 +269,16 @@ static struct gpt_header *get_gpt_header(
 		return NULL;
 	}
 
-	if (le32_to_cpu(h->num_partition_entries) == 0 ||
-	    le32_to_cpu(h->sizeof_partition_entry) == 0 ||
-	    ULONG_MAX / le32_to_cpu(h->num_partition_entries) < le32_to_cpu(h->sizeof_partition_entry)) {
+	/* Size of blocks with GPT entries */
+	esz = (uint64_t)le32_to_cpu(h->num_partition_entries) *
+		le32_to_cpu(h->sizeof_partition_entry);
+
+	if (esz == 0 || esz >= UINT32_MAX ||
+	    le32_to_cpu(h->sizeof_partition_entry) != sizeof(struct gpt_entry)) {
 		DBG(LOWPROBE, ul_debug("GPT entries undefined"));
 		return NULL;
 	}
 
-	/* Size of blocks with GPT entries */
-	esz = le32_to_cpu(h->num_partition_entries) *
-			le32_to_cpu(h->sizeof_partition_entry);
-
 	/* The header seems valid, save it
 	 * (we don't care about zeros in hdr->reserved2 area) */
 	memcpy(hdr, h, sizeof(*h));
@@ -289,7 +293,7 @@ static struct gpt_header *get_gpt_header(
 	}
 
 	/* Validate entries */
-	crc = count_crc32((unsigned char *) *ents, esz);
+	crc = count_crc32((unsigned char *) *ents, esz, 0, 0);
 	if (crc != le32_to_cpu(h->partition_entry_array_crc32)) {
 		DBG(LOWPROBE, ul_debug("GPT entries corrupted"));
 		return NULL;
@@ -317,7 +321,7 @@ static int probe_gpt_pt(blkid_probe pr,
 	ret = is_pmbr_valid(pr, NULL);
 	if (ret < 0)
 		return ret;
-	else if (ret == 0)
+	if (ret == 0)
 		goto nothing;
 
 	errno = 0;
@@ -343,7 +347,7 @@ static int probe_gpt_pt(blkid_probe pr,
 
 	if (blkid_partitions_need_typeonly(pr)) {
 		/* Non-binary interface -- caller does not ask for details
-		 * about partitions, just set generic varibles only. */
+		 * about partitions, just set generic variables only. */
 		blkid_partitions_set_ptuuid(pr, (unsigned char *) &guid);
 		return BLKID_PROBE_OK;
 	}
@@ -392,7 +396,7 @@ static int probe_gpt_pt(blkid_probe pr,
 
 		blkid_partition_set_utf8name(par,
 			(unsigned char *) e->partition_name,
-			sizeof(e->partition_name), BLKID_ENC_UTF16LE);
+			sizeof(e->partition_name), UL_ENCODE_UTF16LE);
 
 		guid = e->unique_partition_guid;
 		swap_efi_guid(&guid);
@@ -423,7 +427,7 @@ const struct blkid_idinfo gpt_pt_idinfo =
 
 	/*
 	 * It would be possible to check for DOS signature (0xAA55), but
-	 * unfortunately almost all EFI GPT implemenations allow to optionaly
+	 * unfortunately almost all EFI GPT implementations allow to optionally
 	 * skip the legacy MBR. We follows this behavior and MBR is optional.
 	 * See is_valid_pmbr().
 	 *
