@@ -74,7 +74,7 @@ static int probe_f2fs(blkid_probe pr, const struct blkid_idmag *mag)
 	if (*((unsigned char *) sb->volume_name))
 		blkid_probe_set_utf8label(pr, (unsigned char *) sb->volume_name,
 						sizeof(sb->volume_name),
-						BLKID_ENC_UTF16LE);
+						UL_ENCODE_UTF16LE);
 
 	blkid_probe_set_uuid(pr, sb->uuid);
 	blkid_probe_sprintf_version(pr, "%u.%u", vermaj, vermin);
