@@ -70,6 +70,18 @@ static struct fdisk_parttype dos_parttypes[] = {
 	#include "pt-mbr-partnames.h"
 };
 
+static const struct fdisk_shortcut dos_parttype_cuts[] =
+{
+	{ .shortcut = "L", .alias = "linux",    .data = "83" },
+	{ .shortcut = "S", .alias = "swap",     .data = "82" },
+	{ .shortcut = "E", .alias = "extended", .data = "05", .deprecated = 1 }, /* collision with 0x0e type */
+	{ .shortcut = "Ex",.alias = "extended", .data = "05" }, /* MBR extended */
+	{ .shortcut = "U", .alias = "uefi",     .data = "EF" }, /* UEFI system */
+	{ .shortcut = "R", .alias = "raid",     .data = "FD" }, /* Linux RAID */
+	{ .shortcut = "V", .alias = "lvm",      .data = "8E" }, /* LVM */
+	{ .shortcut = "X", .alias = "linuxex",  .data = "85" }  /* Linux extended */
+};
+
 #define set_hsc(h,s,c,sector) { \
 		s = sector % cxt->geom.sectors + 1;			\
 		sector /= cxt->geom.sectors;				\
@@ -681,7 +693,7 @@ static int dos_create_disklabel(struct fdisk_context *cxt)
 	/* random disk signature */
 	if (!has_id) {
 		DBG(LABEL, ul_debug("DOS: generate new ID"));
-		random_get_bytes(&id, sizeof(id));
+		ul_random_get_bytes(&id, sizeof(id));
 	}
 
 	if (fdisk_has_protected_bootbits(cxt))
@@ -707,12 +719,12 @@ static int dos_create_disklabel(struct fdisk_context *cxt)
 	return 0;
 }
 
-static int dos_set_disklabel_id(struct fdisk_context *cxt)
+static int dos_set_disklabel_id(struct fdisk_context *cxt, const char *str)
 {
-	char *end = NULL, *str = NULL;
+	char *buf = NULL;
 	unsigned int id, old;
 	struct fdisk_dos_label *l;
-	int rc;
+	int rc = 0;
 
 	assert(cxt);
 	assert(cxt->label);
@@ -722,18 +734,26 @@ static int dos_set_disklabel_id(struct fdisk_context *cxt)
 
 	l = self_label(cxt);
 	old = mbr_get_id(cxt->firstsector);
-	rc = fdisk_ask_string(cxt,
-			_("Enter the new disk identifier"), &str);
-	if (rc)
-		return rc;
 
-	errno = 0;
-	id = strtoul(str, &end, 0);
-	if (errno || str == end || (end && *end)) {
-		fdisk_warnx(cxt, _("Incorrect value."));
-		return -EINVAL;
+	if (!str) {
+		rc = fdisk_ask_string(cxt,
+			_("Enter the new disk identifier"), &buf);
+		str = buf;
+	}
+	if (!rc) {
+		char *end = NULL;
+
+		errno = 0;
+		id = strtoul(str, &end, 0);
+		if (errno || str == end || (end && *end)) {
+			fdisk_warnx(cxt, _("Incorrect value."));
+			rc = -EINVAL;
+		}
 	}
 
+	free(buf);
+	if (rc)
+		return -EINVAL;
 
 	mbr_set_id(cxt->firstsector, id);
 	l->non_pt_changed = 1;
@@ -814,6 +834,12 @@ static int dos_probe_label(struct fdisk_context *cxt)
 	if (!mbr_is_valid_magic(cxt->firstsector))
 		return 0;
 
+	/* ignore disks with FAT */
+	if (cxt->collision &&
+	    (strcmp(cxt->collision, "vfat") == 0 ||
+	     strcmp(cxt->collision, "ntfs") == 0))
+		return 0;
+
 	dos_init(cxt);
 
 	get_partition_table_geometry(cxt, &h, &s);
@@ -962,6 +988,52 @@ static int get_start_from_user(	struct fdisk_context *cxt,
 	return 0;
 }
 
+/* Returns last available sector in the free space pointed to by start. */
+static int find_last_free(
+			struct fdisk_context *cxt,
+			int logical,
+			fdisk_sector_t begin,
+			fdisk_sector_t stop,
+			fdisk_sector_t *result)
+{
+	fdisk_sector_t last = stop;
+
+	size_t i = logical ? 4 : 0;
+
+	for ( ; i < cxt->label->nparts_max; i++) {
+		struct pte *pe = self_pte(cxt, i);
+
+		assert(pe);
+		fdisk_sector_t p_start = get_abs_partition_start(pe);
+		fdisk_sector_t p_end = get_abs_partition_end(pe);
+
+		if (is_cleared_partition(pe->pt_entry))
+			continue;
+		/* count EBR and begin of the logical partition as used area */
+		if (pe->offset)
+			p_start -= cxt->first_lba;
+
+		if ((p_start >= begin && p_start <= last) ||
+		    (p_end >= begin && p_end <= last)) {
+			last = p_start - 1;
+		}
+		if (last < begin) {
+			DBG(LABEL, ul_debug("no free space <%ju,%ju>",
+					(uintmax_t) begin, (uintmax_t) stop));
+			return -ENOSPC;
+		}
+	}
+
+	if (last == begin)
+		last = stop;
+
+	DBG(LABEL, ul_debug("DOS: last free sector  <%ju,%ju>: %ju",
+			(uintmax_t) begin, (uintmax_t) stop, (uintmax_t) last));
+
+	*result = last;
+	return 0;
+}
+
 static int find_last_free_sector_in_range(
 			struct fdisk_context *cxt,
 			int logical,
@@ -1087,18 +1159,7 @@ static int get_disk_ranges(struct fdisk_context *cxt, int logical,
 	return 0;
 }
 
-static int find_last_free_sector(struct fdisk_context *cxt, int logical, fdisk_sector_t *result)
-{
-	fdisk_sector_t first, last;
-	int rc;
-
-	rc = get_disk_ranges(cxt, logical, &first, &last);
-	if (rc)
-		return rc;
-
-	return find_last_free_sector_in_range(cxt, logical, first, last, result);
-}
-
+/* first free sector on disk */
 static int find_first_free_sector(struct fdisk_context *cxt,
 				int logical,
 				fdisk_sector_t start,
@@ -1120,10 +1181,9 @@ static int add_partition(struct fdisk_context *cxt, size_t n,
 	int sys, read = 0, rc, isrel = 0, is_logical;
 	struct fdisk_dos_label *l = self_label(cxt);
 	struct dos_partition *p = self_partition(cxt, n);
-	struct pte *ext_pe = l->ext_offset ? self_pte(cxt, l->ext_index) : NULL;
 	struct fdisk_ask *ask = NULL;
 
-	fdisk_sector_t start, stop = 0, limit, temp;
+	fdisk_sector_t start, stop, limit, temp;
 
 	DBG(LABEL, ul_debug("DOS: adding partition %zu", n));
 
@@ -1137,32 +1197,28 @@ static int add_partition(struct fdisk_context *cxt, size_t n,
 		return -EINVAL;
 	}
 
-	rc = find_last_free_sector(cxt, is_logical, &limit);
+	rc = get_disk_ranges(cxt, is_logical, &start, &stop);
+	if (rc)
+		return rc;
+
+	if (!is_logical && cxt->parent && fdisk_is_label(cxt->parent, GPT))
+		start = 1;		/* Bad boy modifies hybrid MBR */
+
+	rc = find_last_free_sector_in_range(cxt, is_logical, start, stop, &limit);
 	if (rc == -ENOSPC)
 		fdisk_warnx(cxt, _("No free sectors available."));
 	if (rc)
 		return rc;
 
-	if (!is_logical) {
-		if (cxt->parent && fdisk_is_label(cxt->parent, GPT))
-			start = 1;		/* Bad boy modifies hybrid MBR */
-		else {
-			if (cxt->script && pa && fdisk_partition_has_start(pa)
-			    && pa->start < cxt->first_lba
-			    && pa->start >= 1)
-				fdisk_set_first_lba(cxt, 1);
+	if ((is_logical || !cxt->parent || !fdisk_is_label(cxt->parent, GPT))
+	    && cxt->script && pa && fdisk_partition_has_start(pa)
+	    && pa->start >= (is_logical ? l->ext_offset : 1)
+	    && pa->start < start) {
+		fdisk_set_first_lba(cxt, 1);
 
-			start = cxt->first_lba;
-		}
-	} else {
-		assert(ext_pe);
-
-		if (cxt->script && pa && fdisk_partition_has_start(pa)
-		    && pa->start >= l->ext_offset
-		    && pa->start < l->ext_offset + cxt->first_lba)
-			fdisk_set_first_lba(cxt, 1);
-
-		start = l->ext_offset + cxt->first_lba;
+		rc = get_disk_ranges(cxt, is_logical, &start, &stop);
+		if (rc) /* won't happen, but checking to be proper */
+			return rc;
 	}
 
 	/*
@@ -1173,6 +1229,7 @@ static int add_partition(struct fdisk_context *cxt, size_t n,
 
 		temp = start;
 
+		DBG(LABEL, ul_debug("DOS: >>> search for first free from %ju", start));
 		rc = find_first_free_sector(cxt, is_logical, start, &dflt);
 		if (rc == -ENOSPC)
 			fdisk_warnx(cxt, _("No free sectors available."));
@@ -1199,12 +1256,11 @@ static int add_partition(struct fdisk_context *cxt, size_t n,
 			break;
 		if (start >= temp + fdisk_get_units_per_sector(cxt)
 		    && read) {
-			fdisk_info(cxt, _("Sector %llu is already allocated."),
-					temp);
+			if (!pa || !pa->start_follow_default)
+				fdisk_info(cxt, _("Sector %llu is already allocated."), temp);
 			temp = start;
 			read = 0;
-			if (pa && (fdisk_partition_has_start(pa) ||
-				   pa->start_follow_default))
+			if (pa && fdisk_partition_has_start(pa))
 				break;
 		}
 
@@ -1214,6 +1270,30 @@ static int add_partition(struct fdisk_context *cxt, size_t n,
 				return rc;
 			read = 1;
 		}
+		if (pa && fdisk_partition_has_size(pa)) {
+			fdisk_sector_t last;
+
+			rc = find_last_free(cxt, is_logical, start, limit, &last);
+			if (rc == 0 && last - start + 1 < fdisk_partition_get_size(pa)) {
+				DBG(LABEL, ul_debug("DOS: area <%ju,%ju> too small [wanted=%ju aval=%ju]",
+							(uintmax_t) start, (uintmax_t) last,
+							fdisk_partition_get_size(pa),
+							last - start));
+
+				if (fdisk_partition_has_start(pa)
+				    && fdisk_partition_get_start(pa) <= last)
+					rc = -ENOSPC;
+				else {
+					start = last + 1;
+					continue;
+				}
+			}
+			if (rc == -ENOSPC) {
+				fdisk_warnx(cxt, _("No free sectors available."));
+				return rc;
+			}
+		}
+
 	} while (start != temp || !read);
 
 	if (n == 4) {
@@ -1227,7 +1307,11 @@ static int add_partition(struct fdisk_context *cxt, size_t n,
 		struct pte *pe = self_pte(cxt, n);
 
 		assert(pe);
+		assert(start >= cxt->first_lba);
+
 		pe->offset = start - cxt->first_lba;
+		DBG(LABEL, ul_debug("DOS: setting EBR offset to %ju [start=%ju]", pe->offset, start));
+
 		if (pe->offset == l->ext_offset) { /* must be corrected */
 			pe->offset++;
 			if (cxt->first_lba == 1)
@@ -1235,7 +1319,7 @@ static int add_partition(struct fdisk_context *cxt, size_t n,
 		}
 	}
 
-	rc = find_last_free_sector_in_range(cxt, is_logical, start, limit, &stop);
+	rc = find_last_free(cxt, is_logical, start, limit, &stop);
 	if (rc == -ENOSPC)
 		fdisk_warnx(cxt, _("No free sectors available."));
 	if (rc)
@@ -1307,8 +1391,6 @@ static int add_partition(struct fdisk_context *cxt, size_t n,
 	if (isrel && stop - start < (cxt->grain / fdisk_get_sector_size(cxt))) {
 		/* Don't try to be smart on very small partitions and don't align so small sizes */
 		isrel = 0;
-		if (stop > start)
-			stop -= 1;
 		DBG(LABEL, ul_debug("DOS: don't align end of tiny partition [start=%ju, stop=%ju, grain=%lu]",
 			    (uintmax_t)start,  (uintmax_t)stop, cxt->grain));
 	}
@@ -1321,7 +1403,7 @@ static int add_partition(struct fdisk_context *cxt, size_t n,
 		 */
 		stop = fdisk_align_lba_in_range(cxt, stop, start, limit);
 		if (stop > start)
-			stop -= 1;
+			stop -= 1;	/* end one sector before aligned offset */
 		if (stop > limit)
 			stop = limit;
 		DBG(LABEL, ul_debug("DOS: aligned stop: %ju", (uintmax_t) stop));
@@ -1330,7 +1412,6 @@ static int add_partition(struct fdisk_context *cxt, size_t n,
 	set_partition(cxt, n, 0, start, stop, sys, fdisk_partition_is_bootable(pa));
 	if (n > 4) {
 		struct pte *pe = self_pte(cxt, n);
-
 		assert(pe);
 		set_partition(cxt, n - 1, 1, pe->offset, stop,
 					MBR_DOS_EXTENDED_PARTITION, 0);
@@ -1547,6 +1628,7 @@ static int dos_verify_disklabel(struct fdisk_context *cxt)
 		       last[cxt->label->nparts_max];
 	struct dos_partition *p;
 	struct fdisk_dos_label *l = self_label(cxt);
+	int nerrors = 0;
 
 	assert(fdisk_is_label(cxt, DOS));
 
@@ -1558,10 +1640,12 @@ static int dos_verify_disklabel(struct fdisk_context *cxt)
 		if (p && is_used_partition(p) && !IS_EXTENDED(p->sys_ind)) {
 			check_consistency(cxt, p, i);
 			assert(pe);
-			if (get_abs_partition_start(pe) < first[i])
+			if (get_abs_partition_start(pe) < first[i]) {
 				fdisk_warnx(cxt, _(
 					"Partition %zu: bad start-of-data."),
 					 i + 1);
+				nerrors++;
+			}
 
 			check(cxt, i + 1, p->eh, p->es, p->ec, last[i]);
 			total += last[i] + 1 - first[i];
@@ -1576,6 +1660,7 @@ static int dos_verify_disklabel(struct fdisk_context *cxt)
 					fdisk_warnx(cxt, _("Partition %zu: "
 						"overlaps partition %zu."),
 						j + 1, i + 1);
+					nerrors++;
 
 					total += first[i] >= first[j] ?
 						first[i] : first[j];
@@ -1599,28 +1684,37 @@ static int dos_verify_disklabel(struct fdisk_context *cxt)
 			assert(p);
 
 			if (!p->sys_ind) {
-				if (i != 4 || i + 1 < cxt->label->nparts_max)
+				if (i != 4 || i + 1 < cxt->label->nparts_max) {
 					fdisk_warnx(cxt,
 						_("Partition %zu: empty."),
 						i + 1);
+					nerrors++;
+				}
 			} else if (first[i] < l->ext_offset
 				   || last[i] > e_last) {
 
 				fdisk_warnx(cxt, _("Logical partition %zu: "
 					"not entirely in partition %zu."),
 					i + 1, l->ext_index + 1);
+				nerrors++;
 			}
 		}
 	}
 
-	if (total > n_sectors)
-		fdisk_warnx(cxt, _("Total allocated sectors %llu greater "
-			"than the maximum %llu."), total, n_sectors);
-	else if (total < n_sectors)
-		fdisk_warnx(cxt, _("Remaining %lld unallocated %ld-byte "
-			"sectors."), n_sectors - total, cxt->sector_size);
+	if (!nerrors) {
+		fdisk_info(cxt, _("No errors detected."));
+		if (total > n_sectors)
+			fdisk_info(cxt, _("Total allocated sectors %llu greater "
+				"than the maximum %llu."), total, n_sectors);
+		else if (total < n_sectors)
+			fdisk_info(cxt, _("Remaining %lld unallocated %ld-byte "
+				"sectors."), n_sectors - total, cxt->sector_size);
+	} else
+		fdisk_warnx(cxt,
+			P_("%d error detected.", "%d errors detected.", nerrors),
+			nerrors);
 
-	return 0;
+	return nerrors;
 }
 
 /*
@@ -1633,7 +1727,8 @@ static int dos_add_partition(struct fdisk_context *cxt,
 			     struct fdisk_partition *pa,
 			     size_t *partno)
 {
-	size_t i, free_primary = 0, free_sectors = 0;
+	size_t i;
+	uint8_t free_primary = 0, free_sectors = 0;
 	fdisk_sector_t last = 0, grain;
 	int rc = 0;
 	struct fdisk_dos_label *l;
@@ -1647,6 +1742,13 @@ static int dos_add_partition(struct fdisk_context *cxt,
 	DBG(LABEL, ul_debug("DOS: new partition wanted"));
 
 	l = self_label(cxt);
+
+	if (cxt->label->nparts_max >= MAXIMUM_PARTS) {
+		fdisk_warnx(cxt, _("The maximum number of partitions has "
+				  "been created."));
+		return -EINVAL;
+	}
+
 	ext_pe = l->ext_offset ? self_pte(cxt, l->ext_index) : NULL;
 
 	/*
@@ -1707,7 +1809,9 @@ static int dos_add_partition(struct fdisk_context *cxt,
 		if (!ext_pe) {
 			fdisk_warnx(cxt, _("Extended partition does not exists. Failed to add logical partition."));
 			return -EINVAL;
-		} else if (fdisk_partition_has_start(pa)
+		}
+
+		if (fdisk_partition_has_start(pa)
 			   && pa->start < l->ext_offset
 			   && pa->start > get_abs_partition_end(ext_pe)) {
 			DBG(LABEL, ul_debug("DOS: pa template specifies partno>=4, but start out of extended"));
@@ -1725,6 +1829,12 @@ static int dos_add_partition(struct fdisk_context *cxt,
 	grain = cxt->grain > cxt->sector_size ? cxt->grain / cxt->sector_size : 1;
 	last = cxt->first_lba;
 
+	if (cxt->parent && fdisk_is_label(cxt->parent, GPT)) {
+		/* modifying a hybrid MBR, which throws out the rules */
+		grain = 1;
+		last = 1;
+	}
+
 	for (i = 0; i < 4; i++) {
 		struct dos_partition *p = self_partition(cxt, i);
 
@@ -1740,11 +1850,6 @@ static int dos_add_partition(struct fdisk_context *cxt,
 	if (last + grain < cxt->total_sectors - 1)
 		free_sectors = 1;
 
-	if (!free_primary && cxt->label->nparts_max >= MAXIMUM_PARTS) {
-		fdisk_info(cxt, _("The maximum number of partitions has "
-				  "been created."));
-		return -EINVAL;
-	}
 
 	if (!free_primary || !free_sectors) {
 		DBG(LABEL, ul_debug("DOS: primary impossible, add logical"));
@@ -1761,8 +1866,8 @@ static int dos_add_partition(struct fdisk_context *cxt,
 				if (pa && fdisk_partition_has_start(pa)) {
 					fdisk_warnx(cxt, msg);
 					return -EINVAL;
-				} else
-					fdisk_info(cxt, msg);
+				}
+				fdisk_info(cxt, msg);
 			}
 			rc = add_logical(cxt, pa, &res);
 		} else {
@@ -1774,12 +1879,6 @@ static int dos_add_partition(struct fdisk_context *cxt,
 					  "a primary with an extended partition."));
 			return -EINVAL;
 		}
-	} else if (cxt->label->nparts_max >= MAXIMUM_PARTS) {
-		fdisk_info(cxt, _("All logical partitions are in use. "
-				  "Adding a primary partition."));
-		rc = get_partition_unused_primary(cxt, pa, &res);
-		if (rc == 0)
-			rc = add_partition(cxt, res, pa);
 	} else {
 		char hint[BUFSIZ];
 		struct fdisk_ask *ask;
@@ -1801,7 +1900,7 @@ static int dos_add_partition(struct fdisk_context *cxt,
 		fdisk_ask_menu_set_default(ask, free_primary == 1
 						&& !l->ext_offset ? 'e' : 'p');
 		snprintf(hint, sizeof(hint),
-				_("%zu primary, %d extended, %zu free"),
+				_("%u primary, %d extended, %u free"),
 				4 - (l->ext_offset ? 1 : 0) - free_primary,
 				l->ext_offset ? 1 : 0,
 				free_primary);
@@ -2554,8 +2653,12 @@ struct fdisk_label *fdisk_new_dos_label(struct fdisk_context *cxt __attribute__
 	lb->name = "dos";
 	lb->id = FDISK_DISKLABEL_DOS;
 	lb->op = &dos_operations;
+
 	lb->parttypes = dos_parttypes;
 	lb->nparttypes = ARRAY_SIZE(dos_parttypes) - 1;
+	lb->parttype_cuts = dos_parttype_cuts;
+	lb->nparttype_cuts = ARRAY_SIZE(dos_parttype_cuts);
+
 	lb->fields = dos_fields;
 	lb->nfields = ARRAY_SIZE(dos_fields);
 
