@@ -492,7 +492,9 @@ static PyObject *Table_next_fs(TableObject *self)
 	if (rc == 1) {
 		mnt_reset_iter(self->iter, MNT_ITER_FORWARD);
 		Py_RETURN_NONE;
-	} else if (rc)
+	}
+
+	if (rc)
 		return UL_RaiseExc(-rc);
 
 	return PyObjectResultFs(fs);
@@ -733,7 +735,7 @@ PyTypeObject TableType = {
 	sizeof(TableObject), /*tp_basicsize*/
 	0, /*tp_itemsize*/
 	(destructor)Table_destructor, /*tp_dealloc*/
-	NULL, /*tp_print*/
+	0, /*tp_print*/
 	NULL, /*tp_getattr*/
 	NULL, /*tp_setattr*/
 	NULL, /*tp_compare*/
