@@ -484,7 +484,7 @@ int mnt_context_force_unrestricted(struct libmnt_context *cxt)
  * - MNT_OMODE_AUTO is used if nothing else is defined
  * - the flags are evaluated in this order: MNT_OMODE_NOTAB, MNT_OMODE_FORCE,
  *   MNT_OMODE_FSTAB, MNT_OMODE_MTAB and then the mount options from fstab/mtab
- *   are set according to MNT_OMODE_{IGNORE,APPEND,PREPAND,REPLACE}
+ *   are set according to MNT_OMODE_{IGNORE,APPEND,PREPEND,REPLACE}
  *
  * Returns: 0 on success, negative number in case of error.
  */
@@ -900,7 +900,7 @@ int mnt_context_is_loopdel(struct libmnt_context *cxt)
  * @cxt: mount context
  * @fs: filesystem description
  *
- * The mount context uses private @fs by default. This function allows to
+ * The mount context uses private @fs by default. This function can be used to
  * overwrite the private @fs with an external instance. This function
  * increments @fs reference counter (and decrement reference counter of the
  * old fs).
@@ -1094,7 +1094,7 @@ const char *mnt_context_get_fstype(struct libmnt_context *cxt)
  * @cxt: mount context
  * @optstr: comma delimited mount options
  *
- * Note that that MS_MOVE cannot be specified as "string". It's operation that
+ * Note that MS_MOVE cannot be specified as "string". It's operation that
  * is no supported in fstab (etc.)
  *
  * Returns: 0 on success, negative number in case of error.
@@ -1190,7 +1190,7 @@ int mnt_context_set_options_pattern(struct libmnt_context *cxt, const char *patt
  * @tb: fstab
  *
  * The mount context reads /etc/fstab to the private struct libmnt_table by default.
- * This function allows to overwrite the private fstab with an external
+ * This function can be used to overwrite the private fstab with an external
  * instance.
  *
  * This function modify the @tb reference counter. This function does not set
@@ -1493,7 +1493,7 @@ int mnt_context_set_tables_errcb(struct libmnt_context *cxt,
  * @cache: cache instance or NULL
  *
  * The mount context maintains a private struct libmnt_cache by default. This
- * function allows to overwrite the private cache with an external instance.
+ * function can be used to overwrite the private cache with an external instance.
  * This function increments cache reference counter.
  *
  * If the @cache argument is NULL, then the current cache instance is reset.
@@ -1611,7 +1611,7 @@ struct libmnt_lock *mnt_context_get_lock(struct libmnt_context *cxt)
  *
  * Sets mount flags (see mount(2) man page).
  *
- * Note that mount context allows to define mount options by mount flags. It
+ * Note that mount context can be used to define mount options by mount flags. It
  * means you can for example use
  *
  *	mnt_context_set_mflags(cxt, MS_NOEXEC | MS_NOSUID);
@@ -1740,7 +1740,7 @@ int mnt_context_get_user_mflags(struct libmnt_context *cxt, unsigned long *flags
  * @data: mount(2) data
  *
  * The mount context generates mountdata from mount options by default. This
- * function allows to overwrite this behavior, and @data will be used instead
+ * function can be used to overwrite this behavior, and @data will be used instead
  * of mount options.
  *
  * The libmount does not deallocate the data by mnt_free_context(). Note that
@@ -1808,7 +1808,7 @@ int mnt_context_prepare_srcpath(struct libmnt_context *cxt)
 		 * Source is PATH (canonicalize)
 		 */
 		path = mnt_resolve_path(src, cache);
-		if (path && strcmp(path, src))
+		if (path && strcmp(path, src) != 0)
 			rc = mnt_fs_set_source(cxt->fs, path);
 	 }
 
@@ -1855,49 +1855,48 @@ end:
 	return rc;
 }
 
-/* create a mountpoint if X-mount.mkdir[=<mode>] specified */
-static int mkdir_target(const char *tgt, struct libmnt_fs *fs)
+static int is_mkdir_required(const char *tgt, struct libmnt_fs *fs, mode_t *mode, int *rc)
 {
 	char *mstr = NULL;
 	size_t mstr_sz = 0;
-	mode_t mode = 0;
 	struct stat st;
-	int rc;
 
 	assert(tgt);
 	assert(fs);
+	assert(mode);
+	assert(rc);
+
+	*mode = 0;
+	*rc = 0;
 
 	if (mnt_optstr_get_option(fs->user_optstr, "X-mount.mkdir", &mstr, &mstr_sz) != 0 &&
 	    mnt_optstr_get_option(fs->user_optstr, "x-mount.mkdir", &mstr, &mstr_sz) != 0)   	/* obsolete */
 		return 0;
 
-	DBG(CXT, ul_debug("mkdir %s (%s) wanted", tgt, mstr));
-
 	if (mnt_stat_mountpoint(tgt, &st) == 0)
 		return 0;
 
+	DBG(CXT, ul_debug("mkdir %s (%s) wanted", tgt, mstr));
+
 	if (mstr && mstr_sz) {
 		char *end = NULL;
 
 		errno = 0;
-		mode = strtol(mstr, &end, 8);
+		*mode = strtol(mstr, &end, 8);
 
 		if (errno || !end || mstr + mstr_sz != end) {
 			DBG(CXT, ul_debug("failed to parse mkdir mode '%s'", mstr));
-			return -MNT_ERR_MOUNTOPT;
+			*rc = -MNT_ERR_MOUNTOPT;
+			return 0;
 		}
 	}
 
-	if (!mode)
-		mode = S_IRWXU |			/* 0755 */
+	if (!*mode)
+		*mode = S_IRWXU |			/* 0755 */
 		       S_IRGRP | S_IXGRP |
 		       S_IROTH | S_IXOTH;
 
-	rc = mkdir_p(tgt, mode);
-	if (rc)
-		DBG(CXT, ul_debug("mkdir %s failed: %m", tgt));
-
-	return rc;
+	return 1;
 }
 
 int mnt_context_prepare_target(struct libmnt_context *cxt)
@@ -1905,6 +1904,7 @@ int mnt_context_prepare_target(struct libmnt_context *cxt)
 	const char *tgt, *prefix;
 	int rc = 0;
 	struct libmnt_ns *ns_old;
+	mode_t mode = 0;
 
 	assert(cxt);
 	assert(cxt->fs);
@@ -1946,12 +1946,15 @@ int mnt_context_prepare_target(struct libmnt_context *cxt)
 	/* X-mount.mkdir target */
 	if (cxt->action == MNT_ACT_MOUNT
 	    && (cxt->user_mountflags & MNT_MS_XCOMMENT ||
-		cxt->user_mountflags & MNT_MS_XFSTABCOMM)) {
+		cxt->user_mountflags & MNT_MS_XFSTABCOMM)
+	    && is_mkdir_required(tgt, cxt->fs, &mode, &rc)) {
 
 		/* supported only for root or non-suid mount(8) */
-		if (!mnt_context_is_restricted(cxt))
-			rc = mkdir_target(tgt, cxt->fs);
-		else
+		if (!mnt_context_is_restricted(cxt)) {
+			rc = mkdir_p(tgt, mode);
+			if (rc)
+				DBG(CXT, ul_debug("mkdir %s failed: %m", tgt));
+		} else
 			rc = -EPERM;
 	}
 
@@ -2861,7 +2864,9 @@ int mnt_context_is_fs_mounted(struct libmnt_context *cxt,
 		}
 		*mounted = 0;
 		return 0;	/* /proc not mounted */
-	} else if (rc)
+	}
+
+	if (rc)
 		return rc;
 
 	*mounted = __mnt_table_is_fs_mounted(mtab, fs,
