@@ -236,7 +236,7 @@ struct libscols_table {
 			colors_wanted	:1,	/* enable colors */
 			is_term		:1,	/* isatty() */
 			padding_debug	:1,	/* output visible padding chars */
-			is_dummy_print	:1,	/* printing used for width calcualion only */
+			is_dummy_print	:1,	/* printing used for width calculation only */
 			maxout		:1,	/* maximize output */
 			minout		:1,	/* minimize output (mutually exclusive to maxout) */
 			header_repeat   :1,     /* print header after libscols_table->termheight */
