@@ -96,9 +96,6 @@
 #define THREAD_LOCAL static
 #endif
 
-/* index with UUID_VARIANT_xxx and shift 5 bits */
-static unsigned char variant_bits[] = { 0x00, 0x04, 0x06, 0x07 };
-
 #ifdef _WIN32
 static void gettimeofday (struct timeval *tv, void *dummy)
 {
@@ -139,7 +136,7 @@ static int get_node_id(unsigned char *node_id)
 	struct ifconf	ifc;
 	char buf[1024];
 	int		n, i;
-	unsigned char	*a;
+	unsigned char	*a = NULL;
 #ifdef HAVE_NET_IF_DL_H
 	struct sockaddr_dl *sdlp;
 #endif
@@ -197,7 +194,7 @@ static int get_node_id(unsigned char *node_id)
 #endif /* HAVE_NET_IF_DL_H */
 #endif /* SIOCGENADDR */
 #endif /* SIOCGIFHWADDR */
-		if (!a[0] && !a[1] && !a[2] && !a[3] && !a[4] && !a[5])
+		if (a == NULL || (!a[0] && !a[1] && !a[2] && !a[3] && !a[4] && !a[5]))
 			continue;
 		if (node_id) {
 			memcpy(node_id, a, 6);
@@ -278,7 +275,7 @@ static int get_clock(uint32_t *clock_high, uint32_t *clock_low,
 	}
 
 	if ((last.tv_sec == 0) && (last.tv_usec == 0)) {
-		random_get_bytes(&clock_seq, sizeof(clock_seq));
+		ul_random_get_bytes(&clock_seq, sizeof(clock_seq));
 		clock_seq &= 0x3FFF;
 		gettimeofday(&last, NULL);
 		last.tv_sec--;
@@ -416,7 +413,7 @@ int __uuid_generate_time(uuid_t out, int *num)
 
 	if (!has_init) {
 		if (get_node_id(node_id) <= 0) {
-			random_get_bytes(node_id, 6);
+			ul_random_get_bytes(node_id, 6);
 			/*
 			 * Set multicast bit, to prevent conflicts
 			 * with IEEE 802 addresses obtained from
@@ -514,7 +511,7 @@ void __uuid_generate_random(uuid_t out, int *num)
 		n = *num;
 
 	for (i = 0; i < n; i++) {
-		random_get_bytes(buf, sizeof(buf));
+		ul_random_get_bytes(buf, sizeof(buf));
 		uuid_unpack(buf, &uu);
 
 		uu.clock_seq = (uu.clock_seq & 0x3FFF) | 0x8000;
@@ -566,21 +563,22 @@ void uuid_generate_md5(uuid_t out, const uuid_t ns, const char *name, size_t len
 {
 	UL_MD5_CTX ctx;
 	char hash[UL_MD5LENGTH];
+	uuid_t buf;
+	struct uuid uu;
 
 	ul_MD5Init(&ctx);
-	/* hash concatenation of well-known UUID with name */
 	ul_MD5Update(&ctx, ns, sizeof(uuid_t));
 	ul_MD5Update(&ctx, (const unsigned char *)name, len);
-
 	ul_MD5Final((unsigned char *)hash, &ctx);
 
-	memcpy(out, hash, sizeof(uuid_t));
+	assert(sizeof(buf) <= sizeof(hash));
 
-	out[6] &= ~(UUID_TYPE_MASK << UUID_TYPE_SHIFT);
-	out[6] |= (UUID_TYPE_DCE_MD5 << UUID_TYPE_SHIFT);
+	memcpy(buf, hash, sizeof(buf));
+	uuid_unpack(buf, &uu);
 
-	out[8] &= ~(UUID_VARIANT_MASK << UUID_VARIANT_SHIFT);
-	out[8] |= (variant_bits[UUID_VARIANT_DCE] << UUID_VARIANT_SHIFT);
+	uu.clock_seq = (uu.clock_seq & 0x3FFF) | 0x8000;
+	uu.time_hi_and_version = (uu.time_hi_and_version & 0x0FFF) | 0x3000;
+	uuid_pack(&uu, out);
 }
 
 /*
@@ -591,20 +589,20 @@ void uuid_generate_sha1(uuid_t out, const uuid_t ns, const char *name, size_t le
 {
 	UL_SHA1_CTX ctx;
 	char hash[UL_SHA1LENGTH];
+	uuid_t buf;
+	struct uuid uu;
 
 	ul_SHA1Init(&ctx);
-	/* hash concatenation of well-known UUID with name */
 	ul_SHA1Update(&ctx, ns, sizeof(uuid_t));
 	ul_SHA1Update(&ctx, (const unsigned char *)name, len);
-
 	ul_SHA1Final((unsigned char *)hash, &ctx);
 
-	memcpy(out, hash, sizeof(uuid_t));
+	assert(sizeof(buf) <= sizeof(hash));
 
-	out[6] &= ~(UUID_TYPE_MASK << UUID_TYPE_SHIFT);
-	out[6] |= (UUID_TYPE_DCE_SHA1 << UUID_TYPE_SHIFT);
+	memcpy(buf, hash, sizeof(buf));
+	uuid_unpack(buf, &uu);
 
-	out[8] &= ~(UUID_VARIANT_MASK << UUID_VARIANT_SHIFT);
-	out[8] |= (variant_bits[UUID_VARIANT_DCE] << UUID_VARIANT_SHIFT);
+	uu.clock_seq = (uu.clock_seq & 0x3FFF) | 0x8000;
+	uu.time_hi_and_version = (uu.time_hi_and_version & 0x0FFF) | 0x5000;
+	uuid_pack(&uu, out);
 }
-
