@@ -147,7 +147,6 @@ static void dbg(char *fmt, ...)
 #endif
 	va_end(args);
 	fflush(NULL);
-	return;
 }
 
 static int lookup_table(const struct ld_table *tab, const char *str)
@@ -314,7 +313,7 @@ int main(int argc, char **argv)
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
-	atexit(close_stdout);
+	close_stdout_atexit();
 
 	/* parse options */
 	if (argc == 0)
