@@ -493,7 +493,8 @@ static void __attribute__((__noreturn__)) usage(void)
 	" LABEL=<label>           specifies device by filesystem label\n"
 	" UUID=<uuid>             specifies device by filesystem UUID\n"
 	" PARTLABEL=<label>       specifies device by partition label\n"
-	" PARTUUID=<uuid>         specifies device by partition UUID\n"));
+	" PARTUUID=<uuid>         specifies device by partition UUID\n"
+	" ID=<id>                 specifies device by udev hardware ID\n"));
 
 	fprintf(out, _(
 	" <device>                specifies device by path\n"
@@ -709,7 +710,18 @@ int main(int argc, char **argv)
 			mnt_context_enable_rwonly_mount(cxt, TRUE);
 			break;
 		case 'o':
-			append_option(cxt, optarg);
+			/* "move" is not supported as option string in libmount
+			 * to avoid use in fstab */
+			if (mnt_optstr_get_option(optarg, "move", NULL, 0) == 0) {
+				char *o = xstrdup(optarg);
+
+				mnt_optstr_remove_option(&o, "move");
+				if (o && *o)
+					append_option(cxt, o);
+				oper = is_move = 1;
+				free(o);
+			} else
+				append_option(cxt, optarg);
 			break;
 		case 'O':
 			if (mnt_context_set_options_pattern(cxt, optarg))
