@@ -580,7 +580,7 @@ static char *replace_u(char *str, char *username)
 		size_t sz;
 		char *tp, *old = entry;
 
-		if (memcmp(p, "\\u", 2)) {
+		if (memcmp(p, "\\u", 2) != 0) {
 			p++;
 			continue;	/* no \u */
 		}
@@ -700,6 +700,7 @@ static void output_version(void)
 static void parse_args(int argc, char **argv, struct options *op)
 {
 	int c;
+	int opt_show_issue = 0;
 
 	enum {
 		VERSION_OPTION = CHAR_MAX + 1,
@@ -869,8 +870,7 @@ static void parse_args(int argc, char **argv, struct options *op)
 			list_speeds();
 			exit(EXIT_SUCCESS);
 		case ISSUE_SHOW_OPTION:
-			show_issue(op);
-			exit(EXIT_SUCCESS);
+			opt_show_issue = 1;
 			break;
 		case VERSION_OPTION:
 			output_version();
@@ -882,6 +882,11 @@ static void parse_args(int argc, char **argv, struct options *op)
 		}
 	}
 
+	if (opt_show_issue) {
+		show_issue(op);
+		exit(EXIT_SUCCESS);
+	}
+
 	debug("after getopt loop\n");
 
 	if (argc < optind + 1) {
@@ -1279,9 +1284,17 @@ static void termio_init(struct options *op, struct termios *tp)
 		ispeed = cfgetispeed(tp);
 		ospeed = cfgetospeed(tp);
 
-		if (!ispeed) ispeed = TTYDEF_SPEED;
-		if (!ospeed) ospeed = TTYDEF_SPEED;
-
+		/* Save also the original speed to array of the speeds to make
+		 * it possible to return the original after unexpected BREAKs.
+		 */
+		if (op->numspeed)
+			op->speeds[op->numspeed++] = ispeed ? ispeed :
+						     ospeed ? ospeed :
+						     TTYDEF_SPEED;
+		if (!ispeed)
+			ispeed = TTYDEF_SPEED;
+		if (!ospeed)
+			ospeed = TTYDEF_SPEED;
 	} else {
 		ospeed = ispeed = op->speeds[FIRST_SPEED];
 	}
@@ -1701,7 +1714,9 @@ static int wait_for_term_input(int fd)
 		if (FD_ISSET(fd, &rfds)) {
 			return 1;
 
-		} else if (netlink_fd >= 0 && FD_ISSET(netlink_fd, &rfds)) {
+		}
+
+		if (netlink_fd >= 0 && FD_ISSET(netlink_fd, &rfds)) {
 			if (!process_netlink())
 				continue;
 
@@ -1730,7 +1745,7 @@ static int issuedir_filter(const struct dirent *d)
 
 	namesz = strlen(d->d_name);
 	if (!namesz || namesz < ISSUEDIR_EXTSIZ + 1 ||
-	    strcmp(d->d_name + (namesz - ISSUEDIR_EXTSIZ), ISSUEDIR_EXT))
+	    strcmp(d->d_name + (namesz - ISSUEDIR_EXTSIZ), ISSUEDIR_EXT) != 0)
 		return 0;
 
 	/* Accept this */
@@ -1919,24 +1934,32 @@ static void eval_issue_file(struct issue *ie,
 #endif
 	if (!(op->flags & F_ISSUE))
 		goto done;
-
 	/*
-	 * The custom issue file or directory specified by: agetty -f <path>.
+	 * The custom issue file or directory list specified by:
+	 *   agetty --isue-file <path[:path]...>
 	 * Note that nothing is printed if the file/dir does not exist.
 	 */
 	if (op->issue) {
-		struct stat st;
+		char *list = strdup(op->issue);
+		char *file;
 
-		if (stat(op->issue, &st) < 0)
-			goto done;
-		if (S_ISDIR(st.st_mode))
-			issuedir_read(ie, op->issue, op, tp);
-		else
-			issuefile_read(ie, op->issue, op, tp);
+		if (!list)
+			log_err(_("failed to allocate memory: %m"));
+
+		for (file = strtok(list, ":"); file; file = strtok(NULL, ":")) {
+			struct stat st;
+
+			if (stat(file, &st) < 0)
+				continue;
+			if (S_ISDIR(st.st_mode))
+				issuedir_read(ie, file, op, tp);
+			else
+				issuefile_read(ie, file, op, tp);
+		}
+		free(list);
 		goto done;
 	}
 
-
 	/* The default /etc/issue and optional /etc/issue.d directory as
 	 * extension to the file. The /etc/issue.d directory is ignored if
 	 * there is no /etc/issue file. The file may be empty or symlink.
@@ -2248,6 +2271,9 @@ static char *get_logname(struct issue *ie, struct options *op, struct termios *t
 				break;
 			case CTL('D'):
 				exit(EXIT_SUCCESS);
+			case CTL('C'):
+				/* Ignore */
+				break;
 			default:
 				if ((size_t)(bp - logname) >= sizeof(logname) - 1)
 					log_err(_("%s: input overrun"), op->tty);
@@ -2418,7 +2444,7 @@ static void __attribute__((__noreturn__)) usage(void)
 	fputs(_(" -a, --autologin <user>     login the specified user automatically\n"), out);
 	fputs(_(" -c, --noreset              do not reset control mode\n"), out);
 	fputs(_(" -E, --remote               use -r <hostname> for login(1)\n"), out);
-	fputs(_(" -f, --issue-file <file>    display issue file\n"), out);
+	fputs(_(" -f, --issue-file <list>    display issue files or directories\n"), out);
 	fputs(_("     --show-issue           display issue file and exit\n"), out);
 	fputs(_(" -h, --flow-control         enable hardware flow control\n"), out);
 	fputs(_(" -H, --host <hostname>      specify login host\n"), out);
@@ -2721,24 +2747,21 @@ static void output_special_char(struct issue *ie,
 	case 't':
 	{
 		time_t now;
-		struct tm *tm;
+		struct tm tm;
 
 		time(&now);
-		tm = localtime(&now);
-
-		if (!tm)
-			break;
+		localtime_r(&now, &tm);
 
 		if (c == 'd') /* ISO 8601 */
 			fprintf(ie->output, "%s %s %d  %d",
-				      nl_langinfo(ABDAY_1 + tm->tm_wday),
-				      nl_langinfo(ABMON_1 + tm->tm_mon),
-				      tm->tm_mday,
-				      tm->tm_year < 70 ? tm->tm_year + 2000 :
-				      tm->tm_year + 1900);
+				      nl_langinfo(ABDAY_1 + tm.tm_wday),
+				      nl_langinfo(ABMON_1 + tm.tm_mon),
+				      tm.tm_mday,
+				      tm.tm_year < 70 ? tm.tm_year + 2000 :
+				      tm.tm_year + 1900);
 		else
 			fprintf(ie->output, "%02d:%02d:%02d",
-				      tm->tm_hour, tm->tm_min, tm->tm_sec);
+				      tm.tm_hour, tm.tm_min, tm.tm_sec);
 		break;
 	}
 	case 'l':
